import asyncio
import json
import time
from datetime import datetime
from typing import Any

import requests
from pydantic import BaseModel, PrivateAttr
from vals.graphql_client import Client
from vals.graphql_client.pull_run import PullRun
from vals.sdk.exceptions import ValsException
from vals.sdk.types import RunMetadata, RunParameters, RunStatus, TestResult
from vals.sdk.util import _get_auth_token, be_host, fe_host, get_ariadne_client


class Run(BaseModel):
    id: str
    """Unique identifier for the run."""

    test_suite_id: str
    """Unique identifier for the test suite run was created with."""

    test_suite_title: str
    """Title of the test suite run was created with."""

    model: str
    """Model used to perform the run."""

    pass_percentage: float | None
    """Average pass percentage of all tests."""

    pass_rate: float | None
    """Percentage of checks that passed"""

    pass_rate_error: float | None
    """Error margin for pass rate"""

    success_rate: float | None
    """Number of tests where all checks passed"""

    success_rate_error: float | None
    """Error margin for success rate"""

    status: RunStatus
    """Status of the run"""

    archived: bool
    """Whether the run has been archived"""

    text_summary: str
    """Automatically generated summary of common error modes for the run."""

    timestamp: datetime
    """Timestamp of when the run was created."""

    completed_at: datetime | None
    """Timestamp of when the run was completed."""

    parameters: RunParameters
    """Parameters used to create the run."""

    test_results: list[TestResult]
    """List of test results for the run."""

    _client: Client = PrivateAttr(default_factory=get_ariadne_client)

    @staticmethod
    def _create_from_pull_result(run_id: str, result: PullRun) -> "Run":
        """Helper method to create a Run instance from a pull_run query result"""

        # Map maximum_threads to parallelism for backwards compatibility
        parameters_dict = result.run.typed_parameters.model_dump()
        model = parameters_dict.pop("model_under_test", "")
        if "maximum_threads" in parameters_dict:
            parameters_dict["parallelism"] = parameters_dict.pop("maximum_threads")
        parameters = RunParameters(**parameters_dict)

        return Run(
            id=run_id,
            model=model,
            pass_percentage=(
                result.run.pass_percentage * 100
                if result.run.pass_percentage is not None
                else None
            ),
            pass_rate=result.run.pass_rate.value if result.run.pass_rate else None,
            pass_rate_error=(
                result.run.pass_rate.error if result.run.pass_rate else None
            ),
            success_rate=(
                result.run.success_rate.value if result.run.success_rate else None
            ),
            success_rate_error=(
                result.run.success_rate.error if result.run.success_rate else None
            ),
            status=RunStatus(result.run.status),
            archived=result.run.archived,
            text_summary=result.run.text_summary,
            timestamp=result.run.timestamp,
            completed_at=result.run.completed_at,
            parameters=parameters,
            test_results=[
                TestResult.from_graphql(test_result)
                for test_result in result.test_results
            ],
            test_suite_title=result.run.test_suite.title,
            test_suite_id=result.run.test_suite.id,
        )

    @classmethod
    async def list_runs(
        cls,
        limit: int = 25,
        offset: int = 0,
        suite_id: str | None = None,
        show_archived: bool = False,
    ) -> list["RunMetadata"]:
        """List runs associated with this organization"""
        client = get_ariadne_client()
        result = await client.list_runs(
            limit=limit, offset=offset, suite_id=suite_id, archived=show_archived
        )
        return [
            RunMetadata.from_graphql(run) for run in result.runs_with_count.run_results
        ]

    @classmethod
    async def from_id(cls, run_id: str) -> "Run":
        """Pull most recent metadata and test results from the vals servers."""
        client = get_ariadne_client()
        result = await client.pull_run(run_id=run_id)
        return cls._create_from_pull_result(run_id, result)

    @property
    def url(self) -> str:
        return f"{fe_host()}/results/{self.id}"

    def to_dict(self) -> dict[str, Any]:
        """Converts the run to a dictionary."""
        return self.model_dump(exclude_none=True, exclude_defaults=True, mode="json")

    def to_json_file(self, file_path: str) -> None:
        """Converts the run to a JSON file."""
        with open(file_path, "w") as f:
            json.dump(self.to_dict(), f, indent=2)

    async def pull(self) -> None:
        """Update this Run instance with latest data from vals servers."""
        result = await self._client.pull_run(run_id=self.id)
        updated = self._create_from_pull_result(self.id, result)
        # TODO: There's probably a better way to update the object.
        for field in updated.__fields__:
            setattr(self, field, getattr(updated, field))

    async def run_status(self) -> RunStatus:
        """Get the status of a run"""

        result = await self._client.run_status(run_id=self.id)
        self.status = result.run.status
        return RunStatus(result.run.status)

    async def wait_for_run_completion(
        self,
    ) -> RunStatus:
        """
        Block a process until a given run has finished running.

        Returns the status of the run after completion.
        """
        await asyncio.sleep(1)
        status = "in_progress"
        start_time = time.time()
        while status == "in_progress":
            status = await self.run_status()

            # Sleep longer between polls, the longer the run goes.
            if time.time() - start_time < 60:
                sleep_time = 1
            elif time.time() - start_time < 60 * 10:
                sleep_time = 5
            else:
                sleep_time = 10

            await asyncio.sleep(sleep_time)

        return RunStatus(status)

    async def to_csv_string(self) -> str:
        """Same as to_csv, but returns a string instead of writing to a file."""
        response = requests.post(
            url=f"{be_host()}/export_results_to_file/?run_id={self.id}",
            headers={"Authorization": _get_auth_token()},
        )

        if response.status_code != 200:
            raise ValsException("Received Error from Vals Servers: " + response.text)

        return response.text

    async def to_csv(self, file_path: str) -> None:
        """Get the CSV results of a run, as bytes."""
        with open(file_path, "w") as f:
            f.write(await self.to_csv_string())



"""
Typing for the operator string a user can provide. 

This file is autogenerated based on the `operators` GraphQL query. 
"""

from typing import Literal

OperatorType = Literal[
    "includes",
    "paragraph_format",
    "matches_json_schema",
    "satisfies_statement",
    "equals_exactly",
    "legacy_equals",
    "is_not_hallucinating",
    "list_format",
    "answers",
    "not_answers",
    "valid_json",
    "constitutional",
    "no_legal_advice",
    "excludes",
    "consistent_with_context",
    "consistent_with_json",
    "affirmative_answer",
    "golden_check_style",
    "matches_tone",
    "golden_check_content",
    "is_concise",
    "is_not_generated",
    "is_safe",
    "less_than_length",
    "equals",
    "is_coherent",
    "consistent_with_docs",
    "includes_nearly_exactly",
    "negative_answer",
    "logical_includes",
    "includes_exactly",
    "equal_intent",
    "regex",
    "consistent_phrasing",
    "includes_any",
    "safety_consistency",
    "excludes_exactly",
    "progresses_conversation",
    "includes_only",
    "includes_any_v2",
    "valid_yaml",
    "grammar",
    "custom_operator",
    "is_polite",
    "capitalization_check",
    "golden_check_format",
    "matches_format",
]



import hashlib
import os
from io import BytesIO

import httpx
import requests
from gql import Client, gql
from gql.transport.requests import RequestsHTTPTransport
from vals.graphql_client.client import Client as AriadneClient
from vals.sdk.auth import _get_auth_token, _get_region

VALS_ENV = os.getenv("VALS_ENV")


def read_pdf(file: BytesIO):
    """
    Convenience method to parse PDFs to strings
    """
    text = ""
    pdf_reader = PdfReader(file)
    num_pages = len(pdf_reader.pages)
    for page_number in range(num_pages):
        page = pdf_reader.pages[page_number]
        text += page.extract_text()
    return text


def read_docx(file: BytesIO):
    """
    Convenience method to parse docx files to strings
    """
    output = pypandoc.convert_text(file.read(), to="plain", format="docx")
    return output


def be_host():
    region = _get_region()
    if region == "eu-north-1":
        return "https://europebe.playgroundrl.com"
    if VALS_ENV == "LOCAL":
        return "http://localhost:8000"
    if VALS_ENV == "DEV":
        return "https://devbe.playgroundrl.com"

    return "https://prodbe.playgroundrl.com"


def fe_host():
    region = _get_region()
    if region == "eu-north-1":
        return "https://eu.platform.vals.ai"
    if VALS_ENV == "LOCAL":
        return "http://localhost:3000"
    if VALS_ENV == "DEV":
        return "https://dev.platform.vals.ai"

    return "https://platform.vals.ai"


def get_client_legacy():
    """Legacy client, only used for rag suites"""

    transport = RequestsHTTPTransport(
        url=f"{be_host()}/graphql/",
        headers={"Authorization": _get_auth_token()},
        verify=VALS_ENV != "LOCAL",
    )

    client_ = Client(transport=transport, fetch_schema_from_transport=True)
    return client_


def list_rag_suites():
    query = gql(
        f"""
        query getRagSuites {{
            ragSuites {{
            id
            org
            path
            query
            }}
            }}
        """
    )
    response = get_client_legacy().execute(query)

    # TODO: Error check
    return response["ragSuites"]


def get_ariadne_client() -> AriadneClient:
    """
    Use the new codegen-based client
    """
    headers = {"Authorization": _get_auth_token()}
    return AriadneClient(
        url=f"{be_host()}/graphql/",
        http_client=httpx.AsyncClient(headers=headers, timeout=60),
    )


def md5_hash(file) -> str:
    """Produces an md5 hash of the file."""
    hasher = hashlib.md5()
    while chunk := file.read(8192):  # Read in 8 KB chunks
        hasher.update(chunk)
    hash = hasher.hexdigest()
    file.seek(0)
    return hash


def parse_file_id(file_id: str) -> tuple[str, str, str | None]:
    if len(file_id) >= 37 and file_id[-37] == ";":
        # This is a heuristic to check if we are in the old file id regime.
        # I checked and all previously uploaded files should match this
        # Counting the number of slashes ensures that it's not a new file id, that
        # happens to have a semicolon in the wrong spot
        org, rest = file_id.split("/")
        filename, test_suite_id = rest.split(";")
        return org, filename, None

    tokens = file_id.split("/")
    if len(tokens) != 2:
        raise Exception(f"Improperly formatted file_id: {file_id}")

    org, filename_with_hash = tokens

    hash, filename = filename_with_hash.split("-", 1)

    if len(hash) != 32:
        raise Exception(f"Improperly formatted file_id: {file_id}")

    return org, filename, hash


def read_file(file_id: str) -> BytesIO:
    response = requests.post(
        url=f"{be_host()}/download_file/?file_id={file_id}",
        headers={"Authorization": _get_auth_token()},
    )
    return BytesIO(response.content)



# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetTestSuiteData(BaseModel):
    test_suites: List["GetTestSuiteDataTestSuites"] = Field(alias="testSuites")


class GetTestSuiteDataTestSuites(BaseModel):
    description: str
    id: str
    org: str
    title: str
    created: datetime
    global_checks: Any = Field(alias="globalChecks")


GetTestSuiteData.model_rebuild()



import json
from typing import Any, Dict

import click
import requests
from gql import gql
from vals.cli.util import display_error_and_exit, prompt_user_for_rag_suite
from vals.sdk.auth import _get_auth_token
from vals.sdk.util import be_host, list_rag_suites


@click.group(name="rag")
def rag_group():
    """
    Commands relating to starting rag evaluations.
    """
    pass


def create_rag_suite(data: Dict[str, Any]) -> str:
    # Construct the GraphQL mutation string
    query = gql(
        f"""
        mutation createRagSuite {{
            updateRagSuite(
                ragSuiteId: "0",
                query: {json.dumps(data['query'])},
                filePath: {json.dumps(data['id'])}
            ) {{
                ragSuite {{
                    id
                    query
                }}
            }}
        }}
        """
    )
    # Execute the query
    result = get_client().execute(query)
    # Extract the 'id' of the created RagSuite from the result
    suite_id = result["updateRagSuite"]["ragSuite"]["id"]
    return suite_id


@click.command(name="list")
def list_command():
    """
    List rag suites associated with this organization
    """
    suites = list_rag_suites()

    suite_text = "\n".join(
        [f"{i}: {s['id']} {s['query']}" for i, s in enumerate(suites)]
    )
    click.echo(suite_text)


@click.command(name="upload")
@click.argument("file", type=click.STRING, required=True)
@click.argument("query", type=click.STRING, required=True)
def upload_command(file: str, query: str):
    """
    Uploads a CSV file to the server for reranking documents.

    Args:
        file (str): The path to the CSV file. The CSV file should
        have a column named "text" containing the documents to be
        reranked as well as a column named id containing the unique
        identifier for each document.

    Returns:
        str: The file ID that can be used to rerank the documents.
    """

    with open(file, "rb") as f:
        response = requests.post(
            f"{be_host()}/upload_rag_file/",
            files={"file": f},
            headers={"Authorization": _get_auth_token()},
        )
        if response.status_code != 200:
            display_error_and_exit(f"Failed to upload file {file}")

    suite_id = create_rag_suite({"id": response.json()["file_id"], "query": query})

    click.secho(
        "Successfully Uploaded RAG. ID: {}".format(suite_id),
        fg="green",
    )


@click.command(name="rerank")
@click.argument("id", type=click.STRING, required=False)
@click.option("--query", type=click.STRING, default="")
@click.option("--light-ranking", type=click.STRING, default=True)
@click.option("--model", type=click.STRING, default="gpt-3")
def rerank_command(
    id: str = None, query: str = "", light_ranking: bool = True, model: str = "gpt-3"
):
    """
    Reranks the documents in the file based on the provided query.

    Args:
        file_id (str, optional): The ID of the file containing the documents to be reranked. If not provided you will be prompted to choose a rag suite.
        query (str, optional): The query to be used for reranking, if not provided the original query will be used.
        light_ranking (bool, optional): Whether to use light ranking. Defaults to True.
        model (str, optional): The model to be used. Defaults to "gpt-3".

    Returns:
        None. Prints a list of scores that match the order of the documents in the file.
    """
    if id == None:
        id = prompt_user_for_rag_suite()
    q = gql(
        f"""
        query getRagSuites {{
            ragSuites {{
            id
            org
            path
            query
            }}
            }}
        """
    )

    response = get_client().execute(q)

    # TODO: Error check
    file_id, original_query = next(
        (
            (suite["path"], suite["query"])
            for suite in response["ragSuites"]
            if suite["id"] == id
        ),
        None,
    )

    if str(query) == "":
        query = str(original_query)
        click.secho(
            f"Using original query: {query}.",
        )
    else:
        click.secho(
            f"Using custom query: {str(query)}.",
        )

    response = requests.get(
        f"{be_host()}/evaluate_rag_ranking/",
        params={
            "file_id": file_id,
            "query": query,
            "light_ranking": light_ranking,
            "model": model,
        },
        headers={"Authorization": _get_auth_token()},
    )

    if response.status_code != 200:
        display_error_and_exit(f"Failed to run reranking: {file_id}")

    scores = response.json()
    scores["query"] = query

    file_name = f"rag_scores_{id}.json"
    with open(file_name, "w") as f:
        json.dump(scores, f, indent=4)

    click.secho(
        f"Successfully Ran RAG Rankings. Scores saved to {file_name}.",
        fg="green",
    )


rag_group.add_command(upload_command)
rag_group.add_command(rerank_command)
rag_group.add_command(list_command, "list")



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class BatchAddQuestionAnswerPairs(BaseModel):
    batch_add_question_answer_pairs: Optional[
        "BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs"
    ] = Field(alias="batchAddQuestionAnswerPairs")


class BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs(BaseModel):
    question_answer_pairs: List[
        "BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairs"
    ] = Field(alias="questionAnswerPairs")


class BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairs(
    BaseModel
):
    id: Any


BatchAddQuestionAnswerPairs.model_rebuild()
BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs.model_rebuild()



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class UpdateGlobalChecks(BaseModel):
    update_global_checks: Optional["UpdateGlobalChecksUpdateGlobalChecks"] = Field(
        alias="updateGlobalChecks"
    )


class UpdateGlobalChecksUpdateGlobalChecks(BaseModel):
    success: bool


UpdateGlobalChecks.model_rebuild()



# Generated by ariadne-codegen
# Source: http://localhost:8000/graphql/

from enum import Enum


class AppQuestionAnswerSetCreationMethodChoices(str, Enum):
    LIVE_QUERY = "LIVE_QUERY"
    SDK = "SDK"
    UPLOAD = "UPLOAD"


class RunStatus(str, Enum):
    IN_PROGRESS = "IN_PROGRESS"
    ERROR = "ERROR"
    SUCCESS = "SUCCESS"


class TestSuiteSortField(str, Enum):
    TITLE = "TITLE"
    CREATED = "CREATED"
    LAST_MODIFIED_AT = "LAST_MODIFIED_AT"


class SortOrder(str, Enum):
    ASC = "ASC"
    DESC = "DESC"


class RunResultSortField(str, Enum):
    NAME = "NAME"
    STARTED_AT = "STARTED_AT"
    COMPLETED_AT = "COMPLETED_AT"
    PASS_PERCENTAGE = "PASS_PERCENTAGE"


class TierEnum(str, Enum):
    FREE = "FREE"
    STARTUP = "STARTUP"
    ENTERPRISE = "ENTERPRISE"



# Generated by ariadne-codegen

from .add_batch_tests import (
    AddBatchTests,
    AddBatchTestsBatchUpdateTest,
    AddBatchTestsBatchUpdateTestTests,
    AddBatchTestsBatchUpdateTestTestsTestSuite,
)
from .async_base_client import AsyncBaseClient
from .base_model import BaseModel, Upload
from .batch_add_question_answer_pairs import (
    BatchAddQuestionAnswerPairs,
    BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs,
    BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairs,
)
from .client import Client
from .create_or_update_test_suite import (
    CreateOrUpdateTestSuite,
    CreateOrUpdateTestSuiteUpdateTestSuite,
    CreateOrUpdateTestSuiteUpdateTestSuiteTestSuite,
)
from .create_question_answer_set import (
    CreateQuestionAnswerSet,
    CreateQuestionAnswerSetCreateQuestionAnswerSet,
    CreateQuestionAnswerSetCreateQuestionAnswerSetQuestionAnswerSet,
)
from .delete_test_suite import DeleteTestSuite, DeleteTestSuiteDeleteSuite
from .enums import (
    AppQuestionAnswerSetCreationMethodChoices,
    RunResultSortField,
    RunStatus,
    SortOrder,
    TestSuiteSortField,
    TierEnum,
)
from .exceptions import (
    GraphQLClientError,
    GraphQLClientGraphQLError,
    GraphQLClientGraphQLMultiError,
    GraphQLClientHttpError,
    GraphQLClientInvalidResponseError,
)
from .get_operators import GetOperators, GetOperatorsOperators
from .get_test_data import GetTestData, GetTestDataTests, GetTestDataTestsTestSuite
from .get_test_suite_data import GetTestSuiteData, GetTestSuiteDataTestSuites
from .get_test_suites_with_count import (
    GetTestSuitesWithCount,
    GetTestSuitesWithCountTestSuitesWithCount,
    GetTestSuitesWithCountTestSuitesWithCountTestSuites,
    GetTestSuitesWithCountTestSuitesWithCountTestSuitesFolder,
)
from .input_types import (
    CheckInputType,
    CheckModifiersInputType,
    ConditionalCheckInputType,
    ExampleInputType,
    FilterOptionsInput,
    FixedOutputInputType,
    MetadataType,
    ParameterInputType,
    PerCheckHumanReviewInputType,
    QuestionAnswerPairInputType,
    RunResultFilterOptionsInput,
    TestMutationInfo,
)
from .list_runs import (
    ListRuns,
    ListRunsRunsWithCount,
    ListRunsRunsWithCountRunResults,
    ListRunsRunsWithCountRunResultsPassRate,
    ListRunsRunsWithCountRunResultsSuccessRate,
    ListRunsRunsWithCountRunResultsTestSuite,
    ListRunsRunsWithCountRunResultsTypedParameters,
)
from .pull_run import (
    PullRun,
    PullRunRun,
    PullRunRunPassRate,
    PullRunRunSuccessRate,
    PullRunRunTestSuite,
    PullRunRunTypedParameters,
    PullRunTestResults,
    PullRunTestResultsQaPair,
    PullRunTestResultsTest,
)
from .remove_old_tests import RemoveOldTests, RemoveOldTestsRemoveUnusedTests
from .run_param_info import RunParamInfo
from .run_status import RunStatus, RunStatusRun
from .start_run import StartRun, StartRunStartRun
from .update_global_checks import (
    UpdateGlobalChecks,
    UpdateGlobalChecksUpdateGlobalChecks,
)

__all__ = [
    "AddBatchTests",
    "AddBatchTestsBatchUpdateTest",
    "AddBatchTestsBatchUpdateTestTests",
    "AddBatchTestsBatchUpdateTestTestsTestSuite",
    "AppQuestionAnswerSetCreationMethodChoices",
    "AsyncBaseClient",
    "BaseModel",
    "BatchAddQuestionAnswerPairs",
    "BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairs",
    "BatchAddQuestionAnswerPairsBatchAddQuestionAnswerPairsQuestionAnswerPairs",
    "CheckInputType",
    "CheckModifiersInputType",
    "Client",
    "ConditionalCheckInputType",
    "CreateOrUpdateTestSuite",
    "CreateOrUpdateTestSuiteUpdateTestSuite",
    "CreateOrUpdateTestSuiteUpdateTestSuiteTestSuite",
    "CreateQuestionAnswerSet",
    "CreateQuestionAnswerSetCreateQuestionAnswerSet",
    "CreateQuestionAnswerSetCreateQuestionAnswerSetQuestionAnswerSet",
    "DeleteTestSuite",
    "DeleteTestSuiteDeleteSuite",
    "ExampleInputType",
    "FilterOptionsInput",
    "FixedOutputInputType",
    "GetOperators",
    "GetOperatorsOperators",
    "GetTestData",
    "GetTestDataTests",
    "GetTestDataTestsTestSuite",
    "GetTestSuiteData",
    "GetTestSuiteDataTestSuites",
    "GetTestSuitesWithCount",
    "GetTestSuitesWithCountTestSuitesWithCount",
    "GetTestSuitesWithCountTestSuitesWithCountTestSuites",
    "GetTestSuitesWithCountTestSuitesWithCountTestSuitesFolder",
    "GraphQLClientError",
    "GraphQLClientGraphQLError",
    "GraphQLClientGraphQLMultiError",
    "GraphQLClientHttpError",
    "GraphQLClientInvalidResponseError",
    "ListRuns",
    "ListRunsRunsWithCount",
    "ListRunsRunsWithCountRunResults",
    "ListRunsRunsWithCountRunResultsPassRate",
    "ListRunsRunsWithCountRunResultsSuccessRate",
    "ListRunsRunsWithCountRunResultsTestSuite",
    "ListRunsRunsWithCountRunResultsTypedParameters",
    "MetadataType",
    "ParameterInputType",
    "PerCheckHumanReviewInputType",
    "PullRun",
    "PullRunRun",
    "PullRunRunPassRate",
    "PullRunRunSuccessRate",
    "PullRunRunTestSuite",
    "PullRunRunTypedParameters",
    "PullRunTestResults",
    "PullRunTestResultsQaPair",
    "PullRunTestResultsTest",
    "QuestionAnswerPairInputType",
    "RemoveOldTests",
    "RemoveOldTestsRemoveUnusedTests",
    "RunParamInfo",
    "RunResultFilterOptionsInput",
    "RunResultSortField",
    "RunStatus",
    "RunStatus",
    "RunStatusRun",
    "SortOrder",
    "StartRun",
    "StartRunStartRun",
    "TestMutationInfo",
    "TestSuiteSortField",
    "TierEnum",
    "UpdateGlobalChecks",
    "UpdateGlobalChecksUpdateGlobalChecks",
    "Upload",
]



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any

from pydantic import Field

from .base_model import BaseModel


class RunParamInfo(BaseModel):
    run_parameter_info: Any = Field(alias="runParameterInfo")



import click

from .rag import rag_group
from .run import run_group
from .suite import suite_group


@click.group()
def cli():
    pass


cli.add_command(suite_group)
cli.add_command(run_group)
cli.add_command(rag_group)



from functools import wraps
from typing import Callable

from openai import OpenAI

# TODO: Is there a better way to do this besides global variables?
in_tokens = 0
out_tokens = 0


def _wrap_chatcompletion(func: Callable):
    @wraps(func)
    def wrapper(**kwargs):
        response = func(**kwargs)
        global in_tokens, out_tokens
        in_tokens += response.usage.prompt_tokens
        out_tokens += response.usage.completion_tokens

        return response

    return wrapper


# External Facing
def patch(client: OpenAI):
    """
    Calling this function allows the Vals SDK to collect token metadata from any calls to OpenAI
    or models that use the OpenAI API.
    """
    client.chat.completions.create = _wrap_chatcompletion(  # type: ignore
        client.chat.completions.create
    )
    return client



# Generated by ariadne-codegen
# Source: vals/graphql/

from .base_model import BaseModel


class RunStatus(BaseModel):
    run: "RunStatusRun"


class RunStatusRun(BaseModel):
    status: str


RunStatus.model_rebuild()



import sys

import click
from vals.sdk.util import list_rag_suites


def prompt_user_for_rag_suite():
    suites = list_rag_suites()
    click.echo("Rag Suites:")
    click.echo(
        "\n".join([f"{i}: {s['id']} {s['query']}" for i, s in enumerate(suites)])
    )

    idx = click.prompt("Enter the number of the rag suite to run", type=int)
    while not 0 <= idx <= len(suites):
        idx = click.prompt("Invalid choice. Retry", type=int)
    suite_id = suites[idx]["id"]
    return suite_id


def display_error_and_exit(error_msg: str):
    click.secho("ERROR: " + error_msg, fg="red")
    sys.exit(1)


def display_table(
    column_headers: list[str], column_widths: list[int], rows: list[list[str]]
):
    header = (
        "|"
        + "|".join([f" {h:{w}} " for h, w in zip(column_headers, column_widths)])
        + "|"
    )
    click.echo(header)
    seperator_line = (
        "+" + "+".join(["-" * (width + 2) for width in column_widths]) + "+"
    )
    click.echo(seperator_line)
    for row in rows:
        row_str = (
            "| "
            + " | ".join([f"{str(r):{w}}" for r, w in zip(row, column_widths)])
            + " |"
        )
        click.echo(row_str)



import inspect
import json
import os
from time import time
from typing import Any, Callable, cast, overload

import requests
import vals.sdk.patch as patch
from pydantic import BaseModel, PrivateAttr
from tqdm import tqdm
from vals.graphql_client.client import Client
from vals.graphql_client.get_operators import GetOperatorsOperators
from vals.graphql_client.input_types import (
    MetadataType,
    ParameterInputType,
    QuestionAnswerPairInputType,
)
from vals.sdk.run import Run
from vals.sdk.types import (
    Check,
    ModelFunctionType,
    ModelFunctionWithFilesAndContextType,
    QuestionAnswerPair,
    RunParameters,
    SimpleModelFunctionType,
    Test,
    TestSuiteMetadata,
)
from vals.sdk.util import (
    _get_auth_token,
    be_host,
    fe_host,
    get_ariadne_client,
    md5_hash,
    parse_file_id,
    read_file,
)


class Suite(BaseModel):
    id: str | None = None

    title: str
    description: str = ""
    global_checks: list[Check] = []
    tests: list[Test] = []

    """
    Helper to see if we've instantiated the suite, but haven't 
    fetched it from the server yet. 
    """
    _client: Client = PrivateAttr(default_factory=get_ariadne_client)

    @classmethod
    async def list_suites(cls, limit=50, offset=0) -> list[TestSuiteMetadata]:
        """
        Generate a list of all the test suites on the server.

        limit: Total number to return
        offset: Start list at this index.
        """
        client = get_ariadne_client()
        gql_response = await client.get_test_suites_with_count(
            limit=limit,
            offset=offset,
        )
        gql_suites = gql_response.test_suites_with_count.test_suites
        return [TestSuiteMetadata.from_graphql(gql_suite) for gql_suite in gql_suites]

    @classmethod
    async def from_id(cls, suite_id: str) -> "Suite":
        """
        Create a new local test suite based on the data from the server.
        """
        client = get_ariadne_client()
        suites_list = await client.get_test_suite_data(suite_id)
        if len(suites_list.test_suites) == 0:
            raise Exception("Couldn't find suite with id: " + suite_id)

        suite_data = suites_list.test_suites[0]
        title = suite_data.title
        description = suite_data.description

        global_checks = []
        if suite_data.global_checks is not None:
            global_checks = [
                Check.from_graphql(check)
                for check in json.loads(suite_data.global_checks)
            ]

        test_data = await client.get_test_data(suite_id)

        tests = []
        for graphql_test in test_data.tests:
            test = Test.from_graphql_test(graphql_test)
            tests.append(test)

        suite = cls(
            id=suite_id,
            title=title,
            description=description,
            global_checks=global_checks,
            tests=tests,
        )
        return suite

    @classmethod
    async def from_dict(cls, data: dict[str, Any]) -> "Suite":
        """
        Imports the test suite from a dictionary - useful if
        importing from the old format.

        Does not create the suite - you must call suite.create() after construction.
        """

        title = data["title"]
        description = data.get("description", "")
        global_checks = [
            Check.from_graphql(check_dict)
            for check_dict in data.get("global_checks", [])
        ]
        tests = [
            Test(
                input_under_test=test["input_under_test"],
                checks=[
                    Check.from_graphql(check_dict) for check_dict in test["checks"]
                ],
                golden_output=test.get("golden_output", ""),
                tags=test.get("tags", []),
                files_under_test=test.get("files_under_test", []),
                context=test.get("context", {}),
            )
            for test in data["tests"]
        ]
        suite = cls(
            title=title,
            description=description,
            global_checks=global_checks,
            tests=tests,
        )
        await suite._validate_suite()
        return suite

    @classmethod
    async def from_json_file(cls, file_path: str) -> "Suite":
        """
        Imports the test suite from a local JSON file.
        """
        with open(file_path, "r") as f:
            data = json.load(f)
        return await cls.from_dict(data)

    @property
    def url(self):
        return f"{fe_host()}/suites/{self.id}"

    def to_dict(self) -> dict[str, Any]:
        """
        Converts the test suite to a dictionary.
        """
        return self.model_dump(exclude_none=True, exclude_defaults=True)

    def to_json_file(self, file_path: str) -> None:
        """
        Converts the test suite to a JSON file.
        """
        with open(file_path, "w") as f:
            json.dump(self.to_dict(), f, indent=2)

    def to_csv_string(self) -> str:
        """
        Like to_csv_file, but returns the file as a string instead of writing it to a file.
        """
        if not self.id:
            raise Exception("Suite has not been created yet.")

        url = f"{be_host()}/export_tests_to_file/?suite_id={self.id}"
        response = requests.post(
            url,
            headers={"Authorization": _get_auth_token()},
        )
        if response.status_code != 200:
            raise Exception(f"Failed to export tests: {response.text}")

        return response.text

    def to_csv_file(self, file_path: str) -> None:
        """
        Converts the test suite to a CSV file.
        """
        with open(file_path, "w") as f:
            f.write(self.to_csv_string())

    async def create(self, force_creation: bool = False) -> None:
        """
        Creates the test suite on the server.

        force_creation: If True, will create a new test suite, even if it
        already exists.
        """
        if force_creation:
            self.id = None

        if self.id is not None:
            raise Exception("This suite has already been created.")

        await self._validate_suite()

        # Create suite object on the server
        suite = await self._client.create_or_update_test_suite(
            "0", self.title, self.description
        )
        if suite.update_test_suite is None:
            raise Exception("Unable to update the test suite.")
        self.id = suite.update_test_suite.test_suite.id

        await self._upload_global_checks()
        await self._upload_files()
        await self._upload_tests(create_only=True)

    async def delete(self) -> None:
        """
        Deletes the test suite from the server.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet, so there's nothing to delete"
            )

        await self._client.delete_test_suite(self.id)

    async def update(self) -> None:
        """
        Pushes any local changes to the the test suite object to the server.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet, so there's nothing to update"
            )

        await self._client.create_or_update_test_suite(
            self.id, self.title, self.description
        )

        await self._upload_files()
        await self._upload_global_checks()
        await self._upload_tests(create_only=False)

        # Remove any tests that are no longer used after the update.
        await self._client.remove_old_tests(self.id, [test._id for test in self.tests])

    @overload
    async def run(
        self,
        model: str,
        model_name: str = "sdk",
        run_name: str | None = None,
        wait_for_completion: bool = False,
        parameters: RunParameters | None = None,
    ):
        """
        Runs based on a model string (e.g. "gpt-4o").
        """
        pass

    @overload
    async def run(
        self,
        model: ModelFunctionType,
        model_name: str = "sdk",
        run_name: str | None = None,
        wait_for_completion: bool = False,
        parameters: RunParameters | None = None,
    ):
        """
        Runs based on a model function. This can either be a simple model function, which just takes
        an input as a string and produces an output string, or a model function that takes in the input string, a dictionary of filename to file contents, and a context dictionary.
        """
        pass

    @overload
    async def run(
        self,
        model: list[QuestionAnswerPair],
        model_name: str = "sdk",
        run_name: str | None = None,
        wait_for_completion: bool = False,
        parameters: RunParameters | None = None,
    ) -> Run:
        """
        Runs based on on a list of question-answer pairs, which contain inputs and outputs.

        IMPORTANT NOTE: The combined "inputs" of the question-answer pairs (the input under test, the context, and the files) need
        to match exactly the inputs of the tests in the suite, otherwise, Vals will not know how to match to the auto-eval correctly.
        """
        pass

    async def run(
        self,
        model: str | ModelFunctionType | list[QuestionAnswerPair],
        model_name: str = "sdk",
        run_name: str | None = None,
        wait_for_completion: bool = False,
        parameters: RunParameters | None = None,
    ) -> Run:
        """
        Base method for running the test suite. See overloads for documentation.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet. Call suite.create() before calling suite.run()"
            )
        if parameters is None:
            parameters = RunParameters()

        parameter_json = parameters.model_dump()
        del parameter_json[
            "parallelism"
        ]  # Need to explicitly map parallelism to maximum_threads
        parameter_input = ParameterInputType(
            **parameter_json,
            model_under_test="",
            maximum_threads=parameters.parallelism,
        )

        # Collate the local output pairs if we're using a model function.
        qa_set_id = None
        if isinstance(model, Callable):
            # Generate the QA pairs from the model function
            parameter_input.model_under_test = model_name
            qa_pairs = await self._generate_qa_pairs_from_function(model)
            qa_set_id = await self._create_qa_set(
                qa_pairs, parameter_input.model_dump(), model_name
            )
        elif isinstance(model, list):
            # Use the QA pairs we are already provided
            parameter_input.model_under_test = model_name
            qa_set_id = await self._create_qa_set(
                [qa_pair.to_graphql() for qa_pair in model],
                parameter_input.model_dump(),
                model_name,
            )
        elif isinstance(model, str):
            # Just use a model string (e.g. "gpt-4o")
            parameter_input.model_under_test = model
            qa_set_id = None
        else:
            raise Exception(f"Got unexpected type for model: {type(model)}")

        # Start and pull the run.
        response = await self._client.start_run(
            self.id, parameter_input, qa_set_id, run_name
        )
        if response.start_run is None:
            raise Exception("Unable to start the run.")

        run_id = response.start_run.run_id
        run = await Run.from_id(run_id)

        if wait_for_completion:
            await run.wait_for_run_completion()

        await run.pull()

        return run

    async def _upload_files(self):
        """
        Helper method to upload the files to the server.
        Uploads any tests.files_under_test that haven't been uploaded yet.
        """
        if self.id is None:
            raise Exception("This suite has not been created yet.")

        file_name_and_hash_to_file_id = {}

        # First, we populate the dictionary of files that have already been uploaded
        # for this test suite.
        for test in self.tests:
            for file_id in test._file_ids:
                _, name, _hash = parse_file_id(file_id)
                file_name_and_hash_to_file_id[(name, _hash)] = file_id

        # Next, for files we haven't uploaded yet, we upload them
        # and add them to the dictionary
        for test in self.tests:
            if len(test.files_under_test) != 0:
                file_ids = []
                for file_path in test.files_under_test:
                    if not os.path.exists(file_path):
                        raise Exception(f"File does not exist: {file_path}")

                    # First, we compute the filename and the hash of its contents
                    with open(file_path, "rb") as f:
                        file_hash = md5_hash(f)
                    filename = os.path.basename(file_path)
                    name_hash_tuple = (filename, file_hash)

                    # If the file hasn't been uploaded yet, we upload it
                    if name_hash_tuple not in file_name_and_hash_to_file_id:
                        file_id = self._upload_file(self.id, file_path)
                        file_name_and_hash_to_file_id[name_hash_tuple] = file_id

                    # Either way, we add the file id to the test.
                    file_ids.append(file_name_and_hash_to_file_id[name_hash_tuple])

                # Main downside of this approach is that we can only overwrite files
                # we can't add or remove just one file.
                test._file_ids = file_ids

    async def _upload_tests(self, create_only: bool = True) -> None:
        """
        Helper method to upload the tests to the server in batches of 100.
        """
        if self.id is None:
            raise Exception("This suite has not been created yet.")

        # Upload tests in batches of 100
        created_tests = []
        test_mutations = [test.to_test_mutation_info(self.id) for test in self.tests]
        for i in range(0, len(test_mutations), 100):
            batch = test_mutations[i : i + 100]
            batch_result = await self._client.add_batch_tests(
                tests=batch,
                create_only=create_only,
            )
            if batch_result.batch_update_test is None:
                raise Exception("Unable to add tests to the test suite.")
            created_tests.extend(batch_result.batch_update_test.tests)

        # Update the local suite with the new tests to ensure everything is in sync.
        self.tests = [Test.from_graphql_test(test) for test in created_tests]

    async def _validate_checks(
        self, check: Check, operators_dict: dict[str, GetOperatorsOperators]
    ) -> None:
        """Helper method to ensure that operator is correct"""
        if check.operator not in operators_dict:
            raise ValueError(f"Invalid operator: {check.operator}")
        operator = operators_dict[check.operator]
        if not operator.is_unary and (check.criteria is None or check.criteria == ""):
            raise ValueError(
                f"Operator {operator.name_in_doc} is unary, but no criteria was provided."
            )

    async def _validate_suite(self) -> None:
        """Helper method to ensure that all operator strings are correct. Most of the validation is done by Pydantic
        but this handles things Pydantic can't check for, like whether a file exists."""
        operators = (await self._client.get_operators()).operators
        operators_dict = {op.name_in_doc: op for op in operators}

        for test in self.tests:
            for check in test.checks:
                await self._validate_checks(check, operators_dict)

            for file_path in test.files_under_test:
                if not os.path.exists(file_path):
                    raise ValueError(f"File does not exist: {file_path}")
                if not os.path.isfile(file_path):
                    raise ValueError(f"Path is a directory: {file_path}")

        for check in self.global_checks:
            await self._validate_checks(check, operators_dict)

    async def _upload_global_checks(self) -> None:
        """
        Helper method to upload the global checks to the server.
        """
        if self.id is None:
            raise Exception("This suite has not been created yet.")

        await self._client.update_global_checks(
            self.id,
            [gc.to_graphql_input() for gc in self.global_checks],
        )

    async def _generate_qa_pairs_from_function(
        self,
        model_function: ModelFunctionType,
    ) -> list[QuestionAnswerPairInputType]:
        if self.id is None:
            raise Exception(
                "This suite has not been created yet, so we can't create a QA set from it."
            )
        # Pull latest suite data to ensure we're up to date
        updated_suite = await Suite.from_id(self.id)
        for field in self.__fields__:
            setattr(self, field, getattr(updated_suite, field))
        # Inspect the model function to determine if it takes 1 or 3 parameters
        # We dynamically change our call signature based on what the user passes in.
        sig = inspect.signature(model_function)
        num_params = len(sig.parameters)
        if num_params not in [1, 3]:
            raise ValueError("Model function must accept either 1 or 3 parameters")
        is_simple_model_function = num_params == 1

        qa_pairs: list[QuestionAnswerPairInputType] = []
        for test in tqdm(self.tests):
            files = {}
            for file_id in test._file_ids:
                _, file_name, _ = parse_file_id(file_id)
                files[file_name] = read_file(file_id)

            time_start = time()
            in_tokens_start = patch.in_tokens
            out_tokens_start = patch.out_tokens

            if is_simple_model_function:
                casted_model_function = cast(SimpleModelFunctionType, model_function)
                llm_output = casted_model_function(test.input_under_test)
            else:
                casted_model_function = cast(
                    ModelFunctionWithFilesAndContextType, model_function
                )
                llm_output = casted_model_function(
                    test.input_under_test, files, test.context
                )

            time_end = time()
            in_tokens_end = patch.in_tokens
            out_tokens_end = patch.out_tokens

            qa_pairs.append(
                QuestionAnswerPairInputType(
                    input_under_test=test.input_under_test,
                    file_ids=test._file_ids,
                    context=test.context,
                    llm_output=llm_output,
                    metadata=MetadataType(
                        in_tokens=in_tokens_end - in_tokens_start,
                        out_tokens=out_tokens_end - out_tokens_start,
                        duration_seconds=time_end - time_start,
                    ),
                    test_id=test._id,
                )
            )
        return qa_pairs

    async def _create_qa_set(
        self,
        qa_pairs: list[QuestionAnswerPairInputType],
        parameters: dict[str, int | float | str | bool] = {},
        model_under_test: str | None = None,
        batch_size: int = 50,
    ) -> str | None:
        """
        Helper function to create a question-answer set from a model function.
        A question-answer set is just a set of inputs to the LLM, and the LLM's responses.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet, so we can't create a QA set from it."
            )

        response = await self._client.create_question_answer_set(
            self.id,
            [],
            parameters,
            model_under_test or "sdk",
        )
        set_id = response.create_question_answer_set.question_answer_set.id

        for i in range(0, len(qa_pairs), batch_size):
            batch = qa_pairs[i : i + batch_size]
            await self._client.batch_add_question_answer_pairs(set_id, batch)
        if response.create_question_answer_set is None:
            raise Exception("Unable to create the question-answer set.")

        return set_id

    def _upload_file(self, suite_id: str, file_path: str) -> str:
        with open(file_path, "rb") as f:
            response = requests.post(
                f"{be_host()}/upload_file/?test_suite_id={suite_id}",
                files={"file": f},
                headers={"Authorization": _get_auth_token()},
            )
            if response.status_code != 200:
                raise Exception(f"Failed to upload file {file_path}")
            return response.json()["file_id"]



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class DeleteTestSuite(BaseModel):
    delete_suite: Optional["DeleteTestSuiteDeleteSuite"] = Field(alias="deleteSuite")


class DeleteTestSuiteDeleteSuite(BaseModel):
    success: bool


DeleteTestSuite.model_rebuild()



"""
Internal script to regenerate the operator_type.py file. 

Run with python generate_operator_type.py.
"""

import asyncio

from vals.sdk.v2.util import get_ariadne_client


async def generate_operator_type():
    client = get_ariadne_client()
    with open("operator_type_template", "r") as template:
        template_text = template.read()

    operator_list = await client.get_operators()
    list_of_names = [f'"{op.name_in_doc}"' for op in operator_list.operators]

    template_text = template_text.replace("<REPLACE>", ",\n\t".join(list_of_names))

    with open("operator_type.py", "w") as f:

        f.write(template_text)


if __name__ == "__main__":
    asyncio.run(generate_operator_type())



import asyncio
import json
from io import BytesIO
from typing import Any

from vals.sdk.suite import Check, Run, Suite, Test
from vals.sdk.types import QuestionAnswerPair


async def run_with_model_under_test():
    """Run the suite on a stock model, gpt-4o-mini"""
    suite = Suite(
        title="Test Suite",
        global_checks=[Check(operator="grammar")],
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
            ),
        ],
    )
    await suite.create()
    run = await suite.run(model="gpt-4o-mini", wait_for_completion=True)

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")

    # Can save the results to a CSV file.
    await run.to_csv("out.csv")


async def run_with_function():
    """Run the suite on a custom model function."""
    suite = Suite(
        title="Test Suite",
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
            ),
        ],
    )
    await suite.create()

    def function(input_under_test: str) -> str:
        # This would be replaced with your custom model.
        return input_under_test + "!!!"

    run = await suite.run(
        model=function, wait_for_completion=True, model_name="my_function_model"
    )

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")


async def run_with_function_context_and_files():
    """Run the suite with context and files."""
    suite = Suite(
        title="Test Suite",
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
                context={
                    "message_history": [
                        {"role": "user", "content": "What is QSBS?"},
                        {"role": "assistant", "content": "QSBS is a company."},
                    ]
                },
                files_under_test=["data_files/postmoney_safe.docx"],
            ),
        ],
    )
    await suite.create()

    def function(
        input_under_test: str, files: dict[str, BytesIO], context: dict[str, Any]
    ) -> str:
        # Your LLM would leverage the context, the files, and the input_under_test
        # to return a response.
        return input_under_test

    run = await suite.run(
        model=function, wait_for_completion=True, model_name="my_function_model_v2"
    )

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")


async def run_with_qa_pairs():
    """Run the suite with QA pairs."""
    suite = Suite(
        title="Test Suite",
        global_checks=[Check(operator="grammar")],
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
            ),
        ],
    )

    await suite.create()

    qa_pairs = [QuestionAnswerPair(input_under_test="What is QSBS?", llm_output="QSBS")]

    run = await suite.run(
        model=qa_pairs, model_name="test-model", wait_for_completion=True
    )

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")


async def run_with_qa_pairs_and_context():
    """Run the suite with QA pairs and context."""
    context = {
        "message_history": [
            {"role": "user", "content": "What is QSBS?"},
            {"role": "assistant", "content": "QSBS is a company."},
        ]
    }
    suite = Suite(
        title="Test Suite",
        global_checks=[Check(operator="grammar")],
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
                context=context,
            ),
        ],
    )

    await suite.create()

    qa_pairs = [
        QuestionAnswerPair(
            input_under_test="What is QSBS?", llm_output="QSBS", context=context
        )
    ]

    run = await suite.run(
        model=qa_pairs, model_name="test-model", wait_for_completion=True
    )

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")


async def pull_run(run_id: str):
    run = await Run.from_id(run_id)
    print(run.to_dict())


async def all():
    await run_with_model_under_test()
    await run_with_function()
    await run_with_function_context_and_files()
    await run_with_qa_pairs()
    await run_with_qa_pairs_and_context()
    #   await pull_run("19dc86c6-774e-4946-99f4-01ad1bcf4ccf")


if __name__ == "__main__":
    asyncio.run(all())



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetTestData(BaseModel):
    tests: List["GetTestDataTests"]


class GetTestDataTests(BaseModel):
    checks: Any
    test_id: str = Field(alias="testId")
    cross_version_id: str = Field(alias="crossVersionId")
    file_ids: Any = Field(alias="fileIds")
    input_under_test: str = Field(alias="inputUnderTest")
    tags: Any
    context: Any
    golden_output: str = Field(alias="goldenOutput")
    test_suite: "GetTestDataTestsTestSuite" = Field(alias="testSuite")


class GetTestDataTestsTestSuite(BaseModel):
    id: str


GetTestData.model_rebuild()
GetTestDataTests.model_rebuild()



# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class PullRun(BaseModel):
    run: "PullRunRun"
    test_results: List["PullRunTestResults"] = Field(alias="testResults")


class PullRunRun(BaseModel):
    run_id: str = Field(alias="runId")
    pass_percentage: Optional[float] = Field(alias="passPercentage")
    status: str
    text_summary: str = Field(alias="textSummary")
    timestamp: datetime
    completed_at: Optional[datetime] = Field(alias="completedAt")
    archived: bool
    typed_parameters: "PullRunRunTypedParameters" = Field(alias="typedParameters")
    pass_rate: Optional["PullRunRunPassRate"] = Field(alias="passRate")
    success_rate: Optional["PullRunRunSuccessRate"] = Field(alias="successRate")
    test_suite: "PullRunRunTestSuite" = Field(alias="testSuite")


class PullRunRunTypedParameters(BaseModel):
    eval_model: str = Field(alias="evalModel")
    maximum_threads: int = Field(alias="maximumThreads")
    run_golden_eval: bool = Field(alias="runGoldenEval")
    run_confidence_evaluation: bool = Field(alias="runConfidenceEvaluation")
    heavyweight_factor: int = Field(alias="heavyweightFactor")
    create_text_summary: bool = Field(alias="createTextSummary")
    model_under_test: str = Field(alias="modelUnderTest")
    temperature: float
    max_output_tokens: int = Field(alias="maxOutputTokens")
    system_prompt: str = Field(alias="systemPrompt")
    new_line_stop_option: bool = Field(alias="newLineStopOption")


class PullRunRunPassRate(BaseModel):
    value: float
    error: float


class PullRunRunSuccessRate(BaseModel):
    value: float
    error: float


class PullRunRunTestSuite(BaseModel):
    id: str
    title: str


class PullRunTestResults(BaseModel):
    id: str
    llm_output: str = Field(alias="llmOutput")
    pass_percentage: float = Field(alias="passPercentage")
    pass_percentage_with_optional: float = Field(alias="passPercentageWithOptional")
    result_json: Any = Field(alias="resultJson")
    qa_pair: Optional["PullRunTestResultsQaPair"] = Field(alias="qaPair")
    test: "PullRunTestResultsTest"
    metadata: Optional[Any]


class PullRunTestResultsQaPair(BaseModel):
    context: Any
    output_context: Any = Field(alias="outputContext")


class PullRunTestResultsTest(BaseModel):
    test_id: str = Field(alias="testId")
    input_under_test: str = Field(alias="inputUnderTest")
    context: Any


PullRun.model_rebuild()
PullRunRun.model_rebuild()
PullRunTestResults.model_rebuild()



import asyncio
import json
from io import TextIOWrapper

import click
from vals.cli.util import display_table
from vals.sdk.run import Run


@click.group(name="run")
def run_group():
    """
    Commands relating to starting or viewing runs
    """
    pass


async def pull_async(run_id: str, file: TextIOWrapper, csv: bool, _json: bool):
    run = await Run.from_id(run_id)

    if csv:
        file.write(await run.to_csv_string())
    else:
        file.write(json.dumps(run.to_dict(), indent=2))

    click.secho("Successfully pulled run results.", fg="green")


@click.command
@click.argument("run-id", type=click.STRING, required=True)
@click.argument("file", type=click.File("w"), required=True)
@click.option("--csv", is_flag=True, default=False, help="Save as a CSV")
@click.option("--json", is_flag=True, default=False, help="Save as a JSON")
def pull(run_id: str, file: TextIOWrapper, csv: bool, json: bool):
    """
    Pull results of a run and save it to a file.
    """
    asyncio.run(pull_async(run_id, file, csv, json))


async def list_async(
    limit: int, offset: int, suite_id: str | None, show_archived: bool
):
    run_results = await Run.list_runs(
        limit=limit,
        offset=offset - 1,
        show_archived=show_archived,
        suite_id=suite_id,
    )

    column_names = ["#", "Run Name", "Id", "Status", "Pass Rate", "Timestamp"]
    run_name_width = 40
    column_widths = [3, run_name_width, 36, 13, 10, 20]

    rows = []
    for i, run in enumerate(run_results, start=offset):
        date_str = run.timestamp.strftime("%Y/%m/%d %H:%M")
        pass_percentage_str = f"{run.pass_rate:.2f}%" if run.pass_rate else "N/A"
        if len(run.name) > run_name_width:
            run.name = run.name[: run_name_width - 3] + "..."
        rows.append(
            [
                i,
                run.name,
                run.id,
                run.status.value,
                pass_percentage_str,
                date_str,
            ]
        )

    display_table(column_names, column_widths, rows)


@click.command()
@click.option(
    "-l",
    "--limit",
    type=click.INT,
    default=25,
    help="Limit the number of runs to display",
)
@click.option(
    "-o",
    "--offset",
    required=False,
    default=1,
    help="Start table at this row (1-indexed)",
)
@click.option("--suite-id", required=False, help="Filter runs by suite id")
@click.option(
    "--show-archived",
    is_flag=True,
    help="When enabled, archived runs are displayed in the output",
)
def list(limit: int, offset: int, suite_id: str | None, show_archived: bool):
    """
    List runs associated with this organization
    """
    asyncio.run(list_async(limit, offset, suite_id, show_archived))


run_group.add_command(pull)
run_group.add_command(list)
